@@ -21,7 +21,7 @@
 #include "X_farben.h"
 #include "X_grafik.h"
 
-static char sccsid[] = "@(#)X_grafik.c	1.22 5/17/94";
+static char sccsid[] = "@(#)X_grafik.c	1.34 15 Nov 1994";
 
 
 /* private Datenstruktur fuer das 3D-Blickfeld bzw. den Kompass */
@@ -39,9 +39,99 @@ struct karte
 
 static GC farben[FARBENANZ]; /* GCs fuer alle Farben bzw.
                                 schwarz/weiss-Raster */
+static GC text_farbe;
 static int schwarzweiss;     /* Flag, ob die Farbausgabe mit
                                 schwarz/weiss-Rastern erfolgt */
 
+static XFontStruct *text_font;
+
+/* 7-Segment-Darstellung der Ziffern 0 bis 9 und des Leerfeldes */
+static int segment_belegung[11][7] =
+{
+	/* 0 */
+	  1,
+	1,  1,
+	  0,
+	1,  1,
+	  1,
+
+	/* 1 */
+	  0,
+	0,  1,
+	  0,
+	0,  1,
+	  0,
+
+	/* 2 */
+	  1,
+	0,  1,
+	  1,
+	1,  0,
+	  1,
+
+	/* 3 */
+	  1,
+	0,  1,
+	  1,
+	0,  1,
+	  1,
+
+	/* 4 */
+	  0,
+	1,  1,
+	  1,
+	0,  1,
+	  0,
+
+	/* 5 */
+	  1,
+	1,  0,
+	  1,
+	0,  1,
+	  1,
+
+	/* 6 */
+	  1,
+	1,  0,
+	  1,
+	1,  1,
+	  1,
+
+	/* 7 */
+	  1,
+	0,  1,
+	  0,
+	0,  1,
+	  0,
+
+	/* 8 */
+	  1,
+	1,  1,
+	  1,
+	1,  1,
+	  1,
+
+	/* 9 */
+	  1,
+	1,  1,
+	  1,
+	0,  1,
+	  1,
+
+	/* Leerfeld */
+	  0,
+	0,  0,
+	  0,
+	0,  0,
+	  0
+};
+
+/* Beschreibung der sieben Segmente: */
+static int segment_senkrecht[7] = { 0, 1, 1, 0, 1, 1, 0 }; /* senkrechtes
+                                                              Segment? */
+static int segment_x[7] = { 0, 0, 1, 0, 0, 1, 0 }; /* x-Koordinate */
+static int segment_y[7] = { 0, 0, 0, 1, 1, 1, 2 }; /* y-Koordinate */
+
 
 /*
 ** x_skalieren
@@ -325,6 +415,103 @@ XPoint mund[2 * MUNDECKENANZ + 2];
 }
 
 
+/*
+** malkugel
+**  zeichnet eine Kugel (3D-Spieler) mit oder ohne Clipping
+**
+** Parameter:
+**  fenster: Parameter des Fensters, in dem gezeichnet wird
+**  zeichenfeld: Zeichenfeld, in das gezeichnet wird
+**  sichtbar: Liste der sichtbaren Bereiche
+**  sichtanz: Anzahl der sichtbaren Bereiche, -1 fuer kein Clipping
+**  x0: x-Koordinate der linken oberen Ecke des die Kugel umrandenden Rechtecks
+**  y0: y-Koordinate der linken oberen Ecke des die Kugel umrandenden Rechtecks
+**  breite: Breite der Kugel
+**  hoehe: Hoehe der Kugel
+**  blick: Blickrichtung des Spielers; 0 bis TRIGANZ - 1
+**  farbe: Farbe der Kugel
+*/
+static void malkugel(fenster, zeichenfeld, sichtbar, sichtanz, x0, y0,
+	breite, hoehe, blick, farbe)
+struct fenster *fenster;
+Pixmap zeichenfeld;
+XRectangle *sichtbar;
+int sichtanz, x0, y0, breite, hoehe, blick, farbe;
+{
+	GC gc;                             /* GC fuer Clipping */
+	int augelda, augerda, mundda;      /* Flags, welche Teile des
+	                                      Gesichts sichtbar sind */
+	XPoint augel[AUGENECKENANZ];       /* Ecken des linken Auges */
+	XPoint auger[AUGENECKENANZ];       /* Ecken des rechten Auges */
+	XPoint mund[2 * MUNDECKENANZ + 2]; /* Ecken des Mundes */
+
+	/* Kugel nur fuellen, falls nicht transparent */
+	if (farbe != TRANSPARENT)
+	{
+		gc = farben[KUGELFARBEN + farbe - 1];
+
+		if (sichtanz >= 0)
+			/* Clipping aktivieren */
+			XSetClipRectangles(fenster->display, gc, 0, 0,
+				sichtbar, sichtanz, YXBanded);
+
+		/* Ellipse fuellen */
+		XFillArc(fenster->display, zeichenfeld, gc, x0, y0,
+			breite, hoehe, 0, 64 * 360);
+
+		if (sichtanz >= 0)
+			/* Clipping desaktivieren */
+			XSetClipMask(fenster->display, gc, None);
+	}
+
+	/* Ecken der Augen- und Mundpolygone berechnen und
+	   Sichtbarkeitsflags setzen */
+	gesicht_berechnen(&augelda, &augerda, &mundda,
+		augel, auger, mund, x0, y0, breite, hoehe, blick);
+
+	gc = farben[KUGELGFARBEN + farbe];
+
+	if (sichtanz >= 0)
+		/* Clipping aktivieren */
+		XSetClipRectangles(fenster->display, gc, 0, 0,
+			sichtbar, sichtanz, YXBanded);
+
+	/* linkes Auge zeichnen, falls sichtbar */
+	if (augelda)
+		XFillPolygon(fenster->display, zeichenfeld, gc, augel,
+			AUGENECKENANZ, Nonconvex, CoordModeOrigin);
+
+	/* rechtes Auge zeichnen, falls sichtbar */
+	if (augerda)
+		XFillPolygon(fenster->display, zeichenfeld, gc, auger,
+			AUGENECKENANZ, Nonconvex, CoordModeOrigin);
+
+	/* Mund zeichnen, falls sichtbar */
+	if (mundda)
+		XFillPolygon(fenster->display, zeichenfeld, gc, mund,
+			2 * MUNDECKENANZ + 2, Nonconvex, CoordModeOrigin);
+
+	if (sichtanz >= 0)
+		/* Clipping desaktivieren */
+		XSetClipMask(fenster->display, gc, None);
+
+	gc = farben[KUGELRANDFARBE];
+
+	if (sichtanz >= 0)
+		/* Clipping aktivieren */
+		XSetClipRectangles(fenster->display, gc, 0, 0,
+			sichtbar, sichtanz, YXBanded);
+
+	/* Kugelrand als Ellipse zeichnen */
+	XDrawArc(fenster->display, zeichenfeld, gc,
+		x0, y0, breite, hoehe, 0, 64 * 360);
+
+	if (sichtanz >= 0)
+		/* Clipping desaktivieren */
+		XSetClipMask(fenster->display, gc, None);
+}
+
+
 /*
 ** malkugeln
 **  zeichnet alle Kugeln (3D-Spieler) mit Clipping
@@ -341,14 +528,15 @@ Pixmap zeichenfeld;
 int anzkugeln;
 struct kugel *kugeln;
 {
-	int i; /* Index fuer die Kugeln */
-	GC gc; /* benutzt fuer diverse GCs */
+	int i;        /* Index fuer die Kugeln */
+	GC gc;        /* benutzt fuer diverse GCs */
+	int sichtanz; /* Anzahl der sichtbaren Bereiche */
 
 	/* Kugeln rueckwaerts durchlaufen (nach Abstand aufsteigend sortiert) */
 	for (i = anzkugeln - 1; i >= 0; i--)
 	{
-		XRectangle *sichtbar;     /* Liste der sichtbaren Bereiche,
-		                             Koordinaten in Pixeln */
+		XRectangle *sichtbar;      /* Liste der sichtbaren Bereiche,
+		                              Koordinaten in Pixeln */
 		int x0, y0, breite, hoehe; /* linke obere Ecke, Breite und Hoehe
 		                              des (virtuellen) die Kugel umrandenden
 		                              Rechtecks */
@@ -361,16 +549,33 @@ struct kugel *kugeln;
 		{
 			int j; /* Index fuer die sichtbaren Bereiche */
 
-			for (j = 0; j < kugeln[i].sichtanz; j++)
+			for (j = sichtanz = 0; j < kugeln[i].sichtanz; j++)
 			{
+				int breite; /* Zwischenspeicher fuer die Breite des Bereichs */
+
 				/* sichtbaren Bereich berechnen/skalieren */
-				sichtbar[j].x = x_skalieren(kugeln[i].sichtbar[j].x,
+				sichtbar[sichtanz].x = x_skalieren(kugeln[i].sichtbar[j].x,
 					fenster->breite) + 1;
-				sichtbar[j].y = 0;
-				sichtbar[j].width = x_skalieren(kugeln[i].sichtbar[j].x +
+				sichtbar[sichtanz].y = 0;
+				breite = x_skalieren(kugeln[i].sichtbar[j].x +
 					kugeln[i].sichtbar[j].breite, fenster->breite) -
-					sichtbar[j].x;
-				sichtbar[j].height = fenster->hoehe;
+					sichtbar[sichtanz].x;
+				sichtbar[sichtanz].width = breite;
+				sichtbar[sichtanz].height = fenster->hoehe;
+
+				/* faellt der Bereich nicht weg? */
+				if (breite > 0)
+					/* dann zaehlen */
+					sichtanz++;
+			}
+
+			/* keine sichtbaren Bereiche? */
+			if (sichtanz < 1)
+			{
+				/* Speicher fuer das Skalieren der sichtbaren Bereiche wieder
+				   freigeben */
+				speicher_freigeben((void **)&sichtbar);
+				return;
 			}
 		}
 
@@ -387,14 +592,12 @@ struct kugel *kugeln;
 		/* Schatten nur zeichnen, falls Radius (in y-Richtung) > 0 */
 		if (kugeln[i].schattenry > 0)
 		{
-			gc = farben[SCHATTENFARBE];
-
 			/* Clipping aktivieren */
-			XSetClipRectangles(fenster->display, gc,
-				0, 0, sichtbar, kugeln[i].sichtanz, YXBanded);
+			XSetClipRectangles(fenster->display, farben[SCHATTENFARBE],
+				0, 0, sichtbar, sichtanz, YXBanded);
 
 			/* Schatten als gefuellte Ellipse zeichnen */
-			XFillArc(fenster->display, zeichenfeld, gc,
+			XFillArc(fenster->display, zeichenfeld, farben[SCHATTENFARBE],
 				x0,
 				(kugeln[i].schatteny - kugeln[i].schattenry) *
 					fenster->hoehe / FENSTER_HOEHE,
@@ -404,76 +607,12 @@ struct kugel *kugeln;
 				0, 64 * 360);
 
 			/* Clipping desaktivieren */
-			XSetClipMask(fenster->display, gc, None);
+			XSetClipMask(fenster->display, farben[SCHATTENFARBE], None);
 		}
 
-		{
-			int augelda, augerda, mundda;      /* Flags, welche Teile des
-			                                      Gesichts sichtbar sind */
-			XPoint augel[AUGENECKENANZ];       /* Ecken des linken Auges */
-			XPoint auger[AUGENECKENANZ];       /* Ecken des rechten Auges */
-			XPoint mund[2 * MUNDECKENANZ + 2]; /* Ecken des Mundes */
-
-			/* Kugel nur fuellen, falls nicht transparent */
-			if (kugeln[i].farbe != TRANSPARENT)
-			{
-				gc = farben[KUGELFARBEN + kugeln[i].farbe - 1];
-
-				/* Clipping aktivieren */
-				XSetClipRectangles(fenster->display, gc,
-					0, 0, sichtbar, kugeln[i].sichtanz, YXBanded);
-
-				/* Ellipse fuellen */
-				XFillArc(fenster->display, zeichenfeld, gc,
-					x0, y0, breite, hoehe, 0, 64 * 360);
-
-				/* Clipping desaktivieren */
-				XSetClipMask(fenster->display, gc, None);
-			}
-
-			/* Ecken der Augen- und Mundpolygone berechnen und
-			   Sichtbarkeitsflags setzen */
-			gesicht_berechnen(&augelda, &augerda, &mundda,
-				augel, auger, mund,
-				x0, y0, breite, hoehe, kugeln[i].blick);
-
-			gc = farben[KUGELGFARBEN + kugeln[i].farbe];
-
-			/* Clipping aktivieren */
-			XSetClipRectangles(fenster->display, gc,
-				0, 0, sichtbar, kugeln[i].sichtanz, YXBanded);
-
-			/* linkes Auge zeichnen, falls sichtbar */
-			if (augelda)
-				XFillPolygon(fenster->display, zeichenfeld, gc, augel,
-					AUGENECKENANZ, Nonconvex, CoordModeOrigin);
-
-			/* rechtes Auge zeichnen, falls sichtbar */
-			if (augerda)
-				XFillPolygon(fenster->display, zeichenfeld, gc, auger,
-					AUGENECKENANZ, Nonconvex, CoordModeOrigin);
-
-			/* Mund zeichnen, falls sichtbar */
-			if (mundda)
-				XFillPolygon(fenster->display, zeichenfeld, gc, mund,
-					2 * MUNDECKENANZ + 2, Nonconvex, CoordModeOrigin);
-
-			/* Clipping desaktivieren */
-			XSetClipMask(fenster->display, gc, None);
-		}
-
-		gc = farben[KUGELRANDFARBE];
-
-		/* Clipping aktivieren */
-		XSetClipRectangles(fenster->display, gc,
-			0, 0, sichtbar, kugeln[i].sichtanz, YXBanded);
-
-		/* Kugelrand als Ellipse zeichnen */
-		XDrawArc(fenster->display, zeichenfeld, gc,
-			x0, y0, breite, hoehe, 0, 64 * 360);
-
-		/* Clipping desaktivieren */
-		XSetClipMask(fenster->display, gc, None);
+		/* die Kugel mit Clipping zeichnen */
+		malkugel(fenster, zeichenfeld, sichtbar, sichtanz,
+			x0, y0, breite, hoehe, kugeln[i].blick, kugeln[i].farbe);
 
 		/* Speicher fuer das Skalieren der sichtbaren Bereiche wieder
 		   freigeben */
@@ -735,6 +874,80 @@ struct kartenlinie *linien;
 }
 
 
+/*
+** malziffer
+**  zeichnet eine 7-Segment-Ziffer
+**
+** Parameter:
+**  fenster: Parameter des Fensters, in dem gezeichnet wird
+**  zeichenfeld: Zeichenfeld, in das gezeichnet wird
+**  x0: x-Koordinate der linken oberen Ecke
+**  y0: y-Koordinate der linken oberen Ecke
+**  breite: Ziffernbreite
+**  hoehe: Haelfte der Ziffernhoehe
+**  wert: die Ziffer oder 10 fuer Leerfeld
+*/
+static void malziffer(fenster, zeichenfeld, x0, y0, breite, hoehe, wert)
+struct fenster *fenster;
+Pixmap zeichenfeld;
+int x0, y0, breite, hoehe, wert;
+{
+	GC gc;
+	int i; /* Index fuer Segmente */
+
+	for (i = 0; i < 7; i++)
+	{
+		gc = farben[segment_belegung[wert][i] ? RANDFARBE : BODENFARBE];
+
+		if (segment_senkrecht[i])
+			XDrawLine(fenster->display, zeichenfeld, gc,
+				x0 + breite * segment_x[i],
+				y0 + hoehe * segment_y[i] + 1,
+				x0 + breite * segment_x[i],
+				y0 + hoehe * (segment_y[i] + 1) - 1);
+		else
+			XDrawLine(fenster->display, zeichenfeld, gc,
+				x0 + breite * segment_x[i] + 1,
+				y0 + hoehe * segment_y[i],
+				x0 + breite * (segment_x[i] + 1) - 1,
+				y0 + hoehe * segment_y[i]);
+	}
+}
+
+
+/*
+** malzahl
+**  zeichnet eine dreistellige Zahl aus 7-Segment-Ziffern
+**
+** Parameter:
+**  fenster: Parameter des Fensters, in dem gezeichnet wird
+**  zeichenfeld: Zeichenfeld, in das gezeichnet wird
+**  x0: x-Koordinate der linken oberen Ecke
+**  y0: y-Koordinate der linken oberen Ecke
+**  breite: Ziffernbreite
+**  hoehe: Haelfte der Ziffernhoehe
+**  abstand: horizontaler Abstand zwischen den Ziffern
+**  wert: die Zahl
+*/
+static void malzahl(fenster, zeichenfeld, x0, y0, breite, hoehe, abstand, wert)
+struct fenster *fenster;
+Pixmap zeichenfeld;
+int x0, y0, breite, hoehe, abstand, wert;
+{
+	int i; /* Index fuer die Ziffern */
+
+	for (i = 3; i--;)
+	{
+		/* letzte Ziffer zeichnen */
+		malziffer(fenster, zeichenfeld, x0 + (breite + abstand) * i, y0,
+			breite, hoehe, wert || i == 2 ? wert % 10 : 10);
+
+		/* letzte Ziffer wegwerfen */
+		wert /= 10;
+	}
+}
+
+
 /*
 ** bitmap_skalieren
 **  skaliert eine Bitmap von breite * hoehe auf
@@ -839,9 +1052,11 @@ void *daten;
 			0, fenster->hoehe / 2, fenster->breite - 1, fenster->hoehe / 2);
 	}
 	else
+	{
 		/* alles mit BODENFARBE fuellen */
 		XFillRectangle(fenster->display, zeichenfeld, farben[BODENFARBE],
 			0, 0, fenster->breite, fenster->hoehe);
+	}
 
 	/* Waende zeichnen */
 	malwaende(fenster, zeichenfeld, objekte->objekte.anzwaende,
@@ -855,6 +1070,22 @@ void *daten;
 	if (objekte->objekte.hintergrund_zeichnen && objekte->fadenkreuz)
 		zeichne_fadenkreuz(fenster, zeichenfeld);
 
+	if (objekte->objekte.text != NULL)
+	{
+		XCharStruct format;
+		int direction_hint, font_ascent, font_descent;
+		char *text;
+
+		text = objekte->objekte.text;
+
+		XTextExtents(text_font, text, strlen(text), &direction_hint,
+			&font_ascent, &font_descent, &format);
+		XDrawString(fenster->display, zeichenfeld, text_farbe,
+			fenster->breite / 2 - format.width / 2,
+			fenster->hoehe * 15 / 16 + format.ascent / 2, text,
+			strlen(text));
+	}
+
 
 	/* Zeichenfeld in das Fenster kopieren */
 	XCopyArea(fenster->display, zeichenfeld, fenster->window,
@@ -1064,6 +1295,52 @@ void *daten;
 }
 
 
+/*
+** X_zeichne_punkte
+**  zeichnet den Punktestand
+**
+** Parameter:
+**  X_daten: Zeiger auf die private Datenstruktur
+**  fenster: Parameter des Fensters, in dem gezeichnet wird
+**  daten: Zeiger auf eine Struktur mit der Liste der Punktestaende
+**         und Spielerfarben und deren Anzahl
+*/
+void X_zeichne_punkte(X_daten, fenster, daten)
+void *X_daten;
+struct fenster *fenster;
+void *daten;
+{
+	struct blickfeld *blickfeld; /* private Daten */
+	struct punktedaten *punkte;  /* der Punktestand */
+	Pixmap zeichenfeld;          /* Zeichenfeld, in das im Hintergrund
+	                                gezeichnet wird */
+	int i;                       /* Index fuer die Spieler */
+
+	blickfeld = X_daten;
+	punkte = daten;
+	zeichenfeld = blickfeld->zeichenfeld;
+
+	/* Zeichenfeld loeschen */
+	XFillRectangle(fenster->display, zeichenfeld, farben[BODENFARBE],
+		0, 0, fenster->breite, fenster->hoehe);
+
+	for (i = 0; i < punkte->anzpunkte; i++)
+	{
+		/* Gesicht des Spielers zeichnen */
+		malkugel(fenster, zeichenfeld, NULL, -1,
+			5, 5 + 30 * i, 20, 20, TRIGANZ / 2, punkte->punkte[i].farbe);
+
+		/* Punkte des Spielers zeichnen */
+		malzahl(fenster, zeichenfeld, 35, 5 + 30 * i,
+			10, 10, 4, punkte->punkte[i].punkte);
+	}
+
+	/* Zeichenfeld in Fenster kopieren */
+	XCopyArea(fenster->display, zeichenfeld, fenster->window,
+		farben[BODENFARBE], 0, 0, fenster->breite, fenster->hoehe, 0, 0);
+}
+
+
 /*
 ** X_farben_init
 **  initialisiert die globale Farbpalette
@@ -1269,6 +1546,11 @@ int screen;
 		}
 	}
 
+	text_farbe = XCreateGC(display, rootwindow, 0, 0);
+	XSetForeground(display, text_farbe, BlackPixel(display, screen));
+	text_font = XLoadQueryFont(display, "10x20");
+	XSetFont(display, text_farbe, text_font->fid);
+
 	/* kein Fehler aufgetreten */
 	return 0;
 }
