@@ -28,7 +28,7 @@
 #include "netzwerk.h"
 #include "spiel.h"
 
-static char sccsid[] = "@(#)client.c	1.39 6/27/94";
+static char sccsid[] = "@(#)client.c	1.53 23 Nov 1994";
 
 
 /* Breite und Laenge des aktuellen Spielfeldes: */
@@ -51,14 +51,27 @@ static int gegneranz[2];     /* Anzahl der aktuell sichtbaren Mitspieler */
 static int schussanz[2];     /* Anzahl der sichtbaren Mitspieler-Schuesse */
 static int abgeschossen_durch[2]; /* Farbe des Spielers, von dem man
                                      abgeschossen wurde */
+static char *abschuss_spruch[2] = { NULL, NULL }; /* Spruch des Spielers, von
+                                                     dem man abgeschossen
+                                                     wurde */
+
 /* Daten aller sichtbaren Spieler und Schuesse (Position, Farbe, ...): */
 static struct spieler alle_spieler[2][SPIELERANZ];
 static struct schuss alle_schuesse[2][SPIELERANZ];
+
+static int alle_punkte[2][SPIELERANZ];  /* Punktestand aller Spieler; -1 fuer
+                                           Spieler, die nicht existieren */
 static char ereignisse[2][ERG_ANZ];     /* Ereignisse
                                            (siehe ereignisse.h) */
 static char ereignisse_uebrig[ERG_ANZ]; /* noch nicht verarbeitete
                                            Ereignisse (temporaer) */
 
+static int karteneu = 1; /* Flag, muss der Grundriss der Karte neu
+                            gezeichnet werden? */
+
+/* Fehlermeldung und -knopf von milder_fehler (fuer milden_fehler_abfragen) */
+static char **fehlermeldung = NULL, *fehlerknopf = NULL;
+
 
 /*
 ** drehlab
@@ -113,16 +126,24 @@ static void drehlab()
 **
 ** Parameter:
 **  objekte: Zeiger auf die Struktur, die die Listen enthaelt
+**
+** Seiteneffekte:
+**  *objekte wird veraendert
 */
 static void objekt_listen_init(objekte)
 struct objektdaten *objekte;
 {
+	/* Waende- und Kugelliste anlegen */
 	liste_initialisieren(&objekte->anzwaende, (void **)&objekte->waende);
 	liste_initialisieren(&objekte->anzkugeln, (void **)&objekte->kugeln);
+
+	/* Spruch des Gegners loeschen */
+	objekte->text = NULL;
 }
 
 
-/* objekt_listen_freigeben
+/*
+** objekt_listen_freigeben
 **  gibt die Listen mit zu zeichnenden Grafikobjekten wieder frei
 **
 ** Parameter:
@@ -140,6 +161,36 @@ struct objektdaten *objekte;
 		liste_freigeben(&objekte->kugeln[i].sichtanz,
 			(void **)&objekte->kugeln[i].sichtbar);
 	liste_freigeben(&objekte->anzkugeln, (void **)&objekte->kugeln);
+
+	/* Speicher fuer Spruch des Gegners freigeben */
+	if (objekte->text != NULL)
+		speicher_freigeben((void **)&objekte->text);
+}
+
+
+/*
+** punkte_vergleich
+**  vergleicht zwei Punktestaende und gibt zurueck, in welcher Reihenfolge
+**  sie angezeigt werden sollen
+**
+** Parameter:
+**  punkte1: Zeiger auf einen Punktestand
+**  punkte2: Zeiger auf anderen Punktestand
+**
+** Rueckgabewert:
+**  < 0, falls Punktestand 1 vor Punktestand 2
+**  = 0, falls egal
+**  > 0, falls Punktestand 2 vor Punktestand 1
+*/
+static int punkte_vergleich(punkte1, punkte2)
+struct punktestand *punkte1, *punkte2;
+{
+	/* verschieden viele Punkte? */
+	if (punkte1->punkte != punkte2->punkte)
+		return punkte2->punkte - punkte1->punkte;
+
+	/* sonst nach Farbe sortieren */
+	return (int)punkte1->farbe - (int)punkte2->farbe;
 }
 
 
@@ -155,19 +206,24 @@ struct objektdaten *objekte;
 **  schuss_aktiv: Flag, hat der Spieler selbst einen Schuss im Spiel
 **  schussanz: Anzahl der sichtbaren Schuesse der Mitspieler
 **  alle_schuesse: Feld mit den Daten ueber alle sichtbaren Schuesse
-**  abgechossen_durch: Farbe des Spieler, von dem man abgeschossen wurde
+**  abgeschossen_durch: Farbe des Spieler, von dem man abgeschossen wurde
+**  abschuss_spruch: Spruch des Spieler, von dem man abgeschossen wurde
 **  ereignisse: Feld mit den neuen Ereignissen
+**  alle_punkte: Feld mit den Punktestaenden aller Spieler
+**
+** Seiteneffekte:
+**  karteneu wird veraendert
 */
 static void neuaufbau(spieler_aktiv, gegneranz, alle_spieler, schuss_aktiv,
-	schussanz, alle_schuesse, abgeschossen_durch, ereignisse)
+	schussanz, alle_schuesse, abgeschossen_durch, abschuss_spruch, ereignisse,
+	alle_punkte)
 int spieler_aktiv, gegneranz, schuss_aktiv, schussanz;
 struct spieler alle_spieler[SPIELERANZ];
 struct schuss alle_schuesse[SPIELERANZ];
 int abgeschossen_durch;
-char ereignisse[ERG_ANZ];
+char *abschuss_spruch, ereignisse[ERG_ANZ];
+int alle_punkte[SPIELERANZ];
 {
-	static int karteneu = 1;      /* Flag, muss der Grundriss der Karte neu
-	                                 gezeichnet werden? */
 	/* Struktur mit allen Grafik-Daten fuer die 3D-Sicht nach vorn und
 	   Rueckspiegel */
 	struct objektdaten blick_vor, blick_zurueck;
@@ -178,6 +234,8 @@ char ereignisse[ERG_ANZ];
 	struct kartenlinie *linien;   /* Liste mit den Grafik-Daten der Kreise
 	                                 auf der Karte */
 	struct spieler spieler_rueck; /* Daten des Spielers um 180 gedreht */
+	struct punktedaten punkte;    /* Struktur mit den Daten der
+	                                 Punktestaende */
 
 	/* Listen anlegen, in den die Grafik-Daten abgelegt werden */
 	objekt_listen_init(&blick_vor);
@@ -242,6 +300,15 @@ char ereignisse[ERG_ANZ];
 
 		ausschnitt->x = 0;
 		ausschnitt->breite = FENSTER_BREITE;
+
+		/* Abschussspruch des Gegners bekannt? */
+		if (abschuss_spruch != NULL)
+		{
+			/* dann Speicher belegen und kopieren */
+			speicher_belegen((void **)&blick_vor.text,
+				strlen(abschuss_spruch) + 1);
+			strcpy(blick_vor.text, abschuss_spruch);
+		}
 	}
 
 	/* falls Spieler an Leben, Karte mit ihm berechnen */
@@ -253,6 +320,30 @@ char ereignisse[ERG_ANZ];
 		rechne_karte_kreise(&anzkreise, &kreise,
 			gegneranz, &alle_spieler[1]);
 
+	{
+		int i; /* Index fuer Spieler */
+
+		/* Punktestaende in Liste kopieren */
+		liste_initialisieren(&punkte.anzpunkte, (void **)&punkte.punkte);
+		for (i = 0; i < SPIELERANZ; i++)
+			if (alle_punkte[i] >= 0)
+			{
+				struct punktestand *stand; /* temporaerer Zeiger in Liste */
+
+				/* Punktestand und Farbe des Spielers an Liste haengen */
+				liste_verlaengern(&punkte.anzpunkte, (void **)&punkte.punkte,
+					sizeof(struct punktestand));
+				stand = &punkte.punkte[punkte.anzpunkte - 1];
+				stand->punkte = alle_punkte[i]; /* Punkte */
+				stand->farbe = i + 1; /* Farbe */
+			}
+
+		/* Punkte sortieren */
+		if (punkte.anzpunkte)
+			qsort(punkte.punkte, punkte.anzpunkte,
+				sizeof(struct punktestand), punkte_vergleich);
+	}
+
 	/* Zeichenvorgang synchronisieren */
 	zeichne_sync_anfang();
 
@@ -273,6 +364,9 @@ char ereignisse[ERG_ANZ];
 	else
 		zeichne_kompass(-1);
 
+	/* Punktestand zeichnen */
+	zeichne_punkte(&punkte);
+
 	/* audio-visuelle Darstellung der Ereignisse */
 	ereignisse_darstellen(ereignisse);
 
@@ -283,6 +377,7 @@ char ereignisse[ERG_ANZ];
 	objekt_listen_freigeben(&blick_vor);
 	objekt_listen_freigeben(&blick_zurueck);
 	liste_freigeben(&anzkreise, (void **)&kreise);
+	liste_freigeben(&punkte.anzpunkte, (void **)&punkte.punkte);
 
 	if (karteneu)
 	{
@@ -321,18 +416,26 @@ static void client_ende()
 **  ermittelt die Zeiger auf die aktuellen Puffer, in die die neuen
 **  Spieldaten abgelegt werden sollen
 **
-** Parameter : (alles Zeiger auf Zeiger auf die Puffer)
-**  spieler_aktiv_var: Flag, ist der Spieler selbst im Spiel
-**  spieler_var: Daten des Spielers selbst
-**  gegneranz_var: Anzahl der aktuell sichtbaren Mitspieler
-**  gegner_feld: Feld mit den Daten der sichtbaren Mitspieler
-**  schuss_aktiv_var: Flag, hat der Spieler selbst einen Schuss im Spiel
-**  schuss_var: Daten des eigenen Schusses
-**  schussanz_var: Anzahl der sichtbaren Schuesse der Mitspieler
-**  schuesse_feld: Feld mit den Daten der sichtbaren Schuesse der Mitspieler
-**  abgechossen_durch_var: Farbe des Spieler, von dem man abgeschossen wurde
-**  abschluss_spruch_var: Spruch des Spieler, von dem man abgeschossen wurde
-**  ereignisse_feld: Feld mit den neuen Ereignissen
+** Parameter:
+**  spieler_aktiv_var: Zeiger auf Zeiger auf Flag,
+**                     ist der Spieler selbst im Spiel?
+**  spieler_var: Zeiger auf Zeiger auf Daten des Spielers selbst
+**  gegneranz_var: Zeiger auf Zeiger auf Anzahl der aktuell
+**                 sichtbaren Mitspieler
+**  gegner_feld: Zeiger auf Liste mit den Daten der sichtbaren Mitspieler
+**  schuss_aktiv_var: Zeiger auf Zeiger auf Flag, hat der Spieler
+**                    selbst einen Schuss im Spiel?
+**  schuss_var: Zeiger auf Zeiger auf Daten des eigenen Schusses
+**  schussanz_var: Zeiger auf Zeiger auf Anzahl der sichtbaren
+**                 Schuesse der Mitspieler
+**  schuesse_feld: Zeiger auf Liste mit den Daten der sichtbaren
+**                 Schuesse der Mitspieler
+**  abgechossen_durch_var: Zeiger auf Zeiger auf Farbe des Spieler,
+**                         von dem man abgeschossen wurde
+**  abschluss_spruch_var: Zeiger auf Zeiger auf Spruch des Spieler,
+**                        von dem man abgeschossen wurde
+**  ereignisse_feld: Zeiger auf Liste mit den neuen Ereignissen
+**  punkte_feld: Zeiger auf Liste mit den Punktestaenden
 **
 ** Seiteneffekte:
 **  ereignisse_uebrig wird gesetzt
@@ -340,14 +443,19 @@ static void client_ende()
 void spiel_puffer_anlegen(spieler_aktiv_var, spieler_var, gegneranz_var,
 	gegner_feld, schuss_aktiv_var, schuss_var, schussanz_var,
 	schuesse_feld, abgeschossen_durch_var, abschuss_spruch_var,
-	ereignisse_feld)
+	ereignisse_feld, punkte_feld)
 int **spieler_aktiv_var, **gegneranz_var;
 int **schuss_aktiv_var, **schussanz_var;
 struct spieler **spieler_var, **gegner_feld;
 struct schuss **schuss_var, **schuesse_feld;
 int **abgeschossen_durch_var;
 char ***abschuss_spruch_var, **ereignisse_feld;
+int **punkte_feld;
 {
+	/* Speicher fuer alten Spruch freigeben */
+	if (abschuss_spruch[1 - puffer_nr] != NULL)
+		speicher_freigeben((void **)&abschuss_spruch[1 - puffer_nr]);
+
 	/* Zeiger auf die Puffer ermitteln; jeweils der andere als puffer_nr */
 	*spieler_aktiv_var = &spieler_aktiv[1 - puffer_nr];
 	*spieler_var = alle_spieler[1 - puffer_nr];
@@ -361,12 +469,13 @@ char ***abschuss_spruch_var, **ereignisse_feld;
 
 	*abgeschossen_durch_var = &abgeschossen_durch[1 - puffer_nr];
 
-	/* Abschussspruch wird noch nicht ausgewertet */
-	*abschuss_spruch_var = NULL;
+	*abschuss_spruch_var = &abschuss_spruch[1 - puffer_nr];
 
 	*ereignisse_feld = ereignisse[1 - puffer_nr];
 
-	/* falls die vorigen Pufferinhalte noch nicht abgearbeitet wurde,
+	*punkte_feld = alle_punkte[1 - puffer_nr];
+
+	/* falls die vorigen Pufferinhalte noch nicht abgearbeitet wurden,
 	   alle Ereignisse retten */
 	if (puffer_gueltig != puffer_nr)
 	{
@@ -448,10 +557,13 @@ void *bewegung_deskriptor()
 **      oder -1 fuer unbegrenztes Warten
 **  zeichnen: Flag, ob die Grafik neu berechnet und gezeichnet werden soll
 **
+** Rueckgabewert:
+**  1 fuer Fehler, sonst 0
+**
 ** Seiteneffekt:
 **  puffer_nr wird veraendert
 */
-void bewegung(ms, zeichnen)
+int bewegung(ms, zeichnen)
 int ms, zeichnen;
 {
 	{
@@ -459,23 +571,31 @@ int ms, zeichnen;
 
 		/* Rekursion von spiel_paket_erwarten verhindern */
 		if (rekursion)
-			return;
+			/* kein Fehler */
+			return 0;
 
 		rekursion = 1;
 
 		/* Daten vom Server erwarten */
-		spiel_paket_erwarten(ms);
+		if (spiel_paket_erwarten(ms))
+		{
+			rekursion = 0;
+
+			/* Fehler aufgetreten */
+			return 1;
+		}
 
 		rekursion = 0;
 	}
 
 	/* falls nicht gezeichnet werden soll, fertig */
 	if (!zeichnen)
-		return;
+		/* kein Fehler */
+		return 0;
 
 	if (puffer_gueltig == puffer_nr)
-		/* in diesem Fall liegen keine neuen Daten vor */
-		return;
+		/* in diesem Fall liegen keine neuen Daten vor; kein Fehler */
+		return 0;
 
 	/* der andere Puffer enthaelt die neuen Daten */
 	puffer_nr = puffer_gueltig;
@@ -484,17 +604,53 @@ int ms, zeichnen;
 	neuaufbau(spieler_aktiv[puffer_nr], gegneranz[puffer_nr],
 		alle_spieler[puffer_nr], schuss_aktiv[puffer_nr],
 		schussanz[puffer_nr], alle_schuesse[puffer_nr],
-		abgeschossen_durch[puffer_nr], ereignisse[puffer_nr]);
+		abgeschossen_durch[puffer_nr], abschuss_spruch[puffer_nr],
+		ereignisse[puffer_nr], alle_punkte[puffer_nr]);
+
+	/* kein Fehler */
+	return 0;
 }
 
 
 /*
 ** client_spiel_starten
-**  dummy-Routine
-**  wird spaeter aufgerufen, wenn eine neue Spielrunde gestartet wird
+**  wird aufgerufen, um die Verbindung zum Server aufzubauen
+**
+** Parameter:
+**  server_name: Name des Servers
+**  spruch: Spruch des Spielers oder NULL
+**
+** Rueckgabewert:
+**  1 fuer Fehler, sonst 0
+**
+** Seiteneffekte:
+**  Daten werden vom Server empfangen; karteneu wird initialisiert
 */
-void client_spiel_starten()
+int client_spiel_starten(server_name, spruch)
+char *server_name, *spruch;
 {
+	/* Verbindung zum angegebenen Server aufbauen */
+	if (verbindung_aufbauen(server_name))
+		return 1;
+
+	/* Spielparameter / Labyrinth empfangen */
+	if (spielparameter_empfangen("iMaze xview client JC/HUK 1.2",
+		"Anonymous User", strlen(spruch) ? spruch : NULL, &feldbreite,
+		&feldlaenge, &spielfeld))
+		return 1;
+
+	/* Spielfeld drehen */
+	drehlab();
+
+	/* Karte neu aufbauen */
+	karteneu = 1;
+
+	/* jetzt kanns losgehen */
+	if (verbindung_auf_spiel())
+		return 1;
+
+	/* kein Fehler */
+	return 0;
 }
 
 
@@ -531,6 +687,83 @@ char **meldung, *knopf;
 }
 
 
+/*
+** milder_fehler
+**  merkt sich eine Fehlermeldung fuer die Routine milden_fehler_abfragen
+**
+** Parameter:
+**  meldung: Feld von Strings, die die Zeilen des auszugebenden Textes
+**           beinhalten
+**  knopf: Text, der auf dem Knopf stehen soll;
+**         falls NULL, so wird ein Standardtext verwendet
+**
+** Seiteneffekte:
+**  fehlermeldung und fehlerknopf werden gesetzt
+*/
+void milder_fehler(meldung, knopf)
+char **meldung, *knopf;
+{
+	int n; /* Anzahl der Fehlerstrings */
+	int i; /* Index fuer Fehlerstrings */
+
+	/* Speicher fuer vorige Fehlermeldung freigeben */
+	if (fehlermeldung != NULL)
+	{
+		for (i = 0; meldung[i] != NULL; i++)
+			speicher_freigeben((void **)&fehlermeldung[i]);
+		speicher_freigeben((void **)&fehlermeldung);
+	}
+
+	/* Strings zaehlen */
+	for (n = 0; meldung[n] != NULL; n++);
+
+	/* Speicher fuer das Feld belegen und durch Nullzeiger begrenzen */
+	speicher_belegen((void **)&fehlermeldung, (n + 1) * sizeof *fehlermeldung);
+	fehlermeldung[n] = NULL;
+
+	/* Fehlerstrings kopieren */
+	for (i = 0; i < n; i++)
+	{
+		speicher_belegen((void **)&fehlermeldung[i], strlen(meldung[i]) + 1);
+		strcpy(fehlermeldung[i], meldung[i]);
+	}
+
+	/* Speicher fuer vorige Knopfbeschriftung freigeben */
+	if (fehlerknopf != NULL)
+		speicher_freigeben((void **)&fehlerknopf);
+
+	/* Knopfbeschriftung kopieren */
+	if (knopf == NULL)
+		fehlerknopf = NULL;
+	else
+	{
+		speicher_belegen((void **)&fehlerknopf, strlen(knopf) + 1);
+		strcpy(fehlerknopf, knopf);
+	}
+}
+
+
+/*
+** milden_fehler_abfragen
+**  fragt den in milder_fehler gemerkten Fehlertext und -knopf ab
+**
+** Parameter:
+**  meldung: Zeiger auf Feld von Strings, das die Zeilen des
+**           auszugebenden Textes beinhalten soll
+**  knopf: Zeiger auf Text, der auf dem Knopf stehen soll
+**
+** Seiteneffekte:
+**  *meldung und *knopf werden gesetzt
+*/
+void milden_fehler_abfragen(meldung, knopf)
+char ***meldung, **knopf;
+{
+	/* Rueckgabewerte setzen */
+	*meldung = fehlermeldung;
+	*knopf = fehlerknopf;
+}
+
+
 /*
 ** main
 **  die Hauptroutine
@@ -553,26 +786,12 @@ char **argv;
 		/* bei Fehler beenden */
 		exit(1);
 
-	/* falls mehr als eine Option angegeben wurde, Usage ausgeben */
-	if (argc > 2)
+	/* falls eine unbekannte Option angegeben wurde, Fehler melden */
+	if (argc > 1)
 	{
-		fprintf(stderr,
-			"usage: %s [network-options] [graphic-options] [server-host]\n",
-			argv[0]);
+		fprintf(stderr, "%s: unknown option \"%s\"\n", argv[0], argv[1]);
 		exit(1);
 	}
-	/* Verbindung zum angegebenen Server aufbauen */
-	verbindung_aufbauen(argc < 2 ? NULL : argv[1]);
-
-	/* Spielparameter / Labyrinth empfangen */
-	spielparameter_empfangen("iMaze xview client JC/HUK 1.1",
-		&feldbreite, &feldlaenge, &spielfeld);
-
-	/* Spielfeld drehen */
-	drehlab();
-
-	/* jetzt kanns losgehen */
-	verbindung_auf_spiel();
 
 	/* Hauptschleife */
 	grafik_schleife();
