@@ -10,7 +10,7 @@
 */
 
 
-static char sccsid_fehler[] = "@(#)fehler.h	1.4 5/17/94";
+static char sccsid_fehler[] = "@(#)fehler.h	1.5 22 Nov 1994";
 
 
 /* Prototypen fuer globale Grafik-Funktionen */
@@ -18,9 +18,11 @@ static char sccsid_fehler[] = "@(#)fehler.h	1.4 5/17/94";
 #ifdef __STDC__
 
 void uebler_fehler(char **meldung, char *knopf);
+void milder_fehler(char **meldung, char *knopf);
 
 #else
 
 void uebler_fehler();
+void milder_fehler();
 
 #endif
