@@ -12,7 +12,7 @@
 
 #include <sys/types.h> /* fuer u_int... */
 
-static char sccsid_global[] = "@(#)global.h	1.8 5/17/94";
+static char sccsid_global[] = "@(#)global.h	1.8 17 May 1994";
 
 
 /* zusaetzliche Grundfunktionen */
