@@ -23,7 +23,7 @@
 #include "server.h"
 #include "init_spieler.h"
 
-static char sccsid[] = "@(#)server.c	1.57 6/3/94";
+static char sccsid[] = "@(#)server.c	1.60 24 Oct 1994";
 
 
 /* Identifikationstext, mit dem jede Labyrinthdatei beginnt: */
@@ -48,8 +48,11 @@ static void *session_deskriptor; /* Deskriptor zur Kommunikation mit der
 static void ende_handler(signum)
 int signum;
 {
-	/* NT_SPIEL_BEENDEN an Session-Prozess senden */
-	nachricht_senden(session_deskriptor, NULL, NT_SPIEL_BEENDEN);
+	char daten; /* Puffer fuer die Nachricht */
+
+	/* Nachricht NT_SPIEL_BEENDEN an Session-Prozess senden */
+	daten = NT_SPIEL_BEENDEN;
+	nachricht_senden(session_deskriptor, NULL, sizeof daten, &daten);
 }
 
 
@@ -75,6 +78,9 @@ static void server()
 		{
 			void *verbindung; /* Deskriptor fuer die Verbindung zum
 			                     neuen Client */
+			char *spruch;     /* Spruch des neuen Spielers */
+			char *daten;      /* Puffer zum Uebertragen des Spruchs
+			                     an den Sessions-Prozess */
 
 			/* versuchen, die Verbindung entgegenzunehmen */
 			if ((verbindung = verbindung_annehmen()) == NULL)
@@ -88,10 +94,14 @@ static void server()
 
 			/* dem Client die Initialisierungsdaten senden
 			   und die Verbindung vom Prolog auf das Spiel schalten */
-			if (init_spieler(verbindung, feldbreite,
-				feldlaenge, spielfeld) ||
+			if (init_spieler(verbindung, feldbreite, feldlaenge,
+				spielfeld, &spruch) ||
 				verbindung_auf_spiel(session_deskriptor, verbindung))
 			{
+				/* Speicher fuer Spruch bei Fehler wieder freigeben */
+				if (spruch != NULL)
+					speicher_freigeben((void **)&spruch);
+
 				/* bei einem Fehler die Verbindung abbrechen und weiter
 				   warten; einen evtl. gestarteten Initialisierungsprozess
 				   implizit beenden */
@@ -99,10 +109,31 @@ static void server()
 				continue;
 			}
 
+			/* Speicher fuer NT_NEUER_SPIELER und den Spruch belegen */
+			speicher_belegen((void **)&daten,
+				spruch == NULL ? 2 : strlen(spruch) + 2);
+
+			/* die Nachricht beginnt mit NT_NEUER_SPIELER */
+			daten[0] = NT_NEUER_SPIELER;
+
+			/* falls Spruch bekannt, an die Nachricht anhaengen */
+			if (spruch == NULL)
+				daten[1] = 0;
+			else
+			{
+				strcpy(daten + 1, spruch);
+
+				/* Speicher fuer Spruch wieder freigeben */
+				speicher_freigeben((void **)&spruch);
+			}
+
 			/* dem Session-Prozess den Verbindungsdeskriptor per Nachricht
 			   uebergeben */
 			nachricht_senden(session_deskriptor, verbindung,
-				NT_NEUER_SPIELER);
+				strlen(daten + 1) + 1, daten);
+
+			/* Speicher fuer die Nachricht wieder freigeben */
+			speicher_freigeben((void **)&daten);
 
 			/* der Verbindungsdeskriptor wird vom Server nicht mehr
 			   benoetigt, aufraeumen; einen evtl. gestarteten
@@ -112,15 +143,34 @@ static void server()
 		else
 		/* eine Nachricht von einer Session liegt evtl. an */
 		{
-			char typ;         /* Typ der Nachricht; siehe server.h */
+			void *daten;      /* Datenteil der Nachricht */
+			int laenge;       /* Laenge des Datenteils */
 			void *verbindung; /* Verbindungsdeskriptor, auf den die
 			                     Nachricht sich bezieht */
+			char typ;         /* Typ der Nachricht; siehe server.h */
 
 			/* Nachricht (Typ und Deskriptor) entgegennehmen */
-			if (nachricht_empfangen(session_deskriptor, &verbindung, &typ))
+			if (nachricht_empfangen(session_deskriptor, &verbindung,
+				&laenge, &daten))
 				/* keine Nachricht bekommen; weiter warten */
 				continue;
 
+			/* Datenteil besteht nicht genau aus dem Typ? */
+			if (laenge != 1)
+			{
+				/* dann Speicher wieder freigeben */
+				if (laenge)
+					speicher_freigeben(&daten);
+
+				/* und Nachricht ignorieren */
+				continue;
+			}
+
+			typ = *(char *)daten; /* Typ der Nachricht */
+
+			/* Speicher wieder freigeben */
+			speicher_freigeben(&daten);
+
 			/* wenn der Session-Prozess sich beendet hat, den
 			   Server-Prozess auch beenden */
 			if (typ == NT_SPIEL_ENDE)
