@@ -10,7 +10,7 @@
 */
 
 
-static char sccsid_server_netz[] = "@(#)server_netz.h	1.13 6/3/94";
+static char sccsid_server_netz[] = "@(#)server_netz.h	1.14 01 Oct 1994";
 
 
 /* Prototypen fuer globale Netzwerk-Funktionen */
@@ -29,8 +29,10 @@ void verbindung_abbrechen(void *verbindung);
 int verbindung_auf_spiel(void *session, void *verbindung);
 void netzwerk_init(int *argc, char **argv);
 void *session_starten(void);
-void nachricht_senden(void *session, void *verbindung, int typ);
-int nachricht_empfangen(void *session, void **verbindung, char *typ);
+void nachricht_senden(void *session, void *verbindung, int daten_laenge,
+	void *daten);
+int nachricht_empfangen(void *session, void **verbindung, int *daten_laenge,
+	void **daten);
 void nachricht_erwarten(void *session);
 void *nachricht_oder_verbindung_erwarten(int anzsessions, void **sessions);
 
