@@ -10,7 +10,7 @@
 */
 
 
-static char sccsid_signale[] = "@(#)signale.h	1.9 5/26/94";
+static char sccsid_signale[] = "@(#)signale.h	1.9 26 May 1994";
 
 
 #define AKTIONSSIGNALANZ 10 /* Anzahl der frei verfuegbaren Signale
