@@ -13,6 +13,7 @@
 #include <xview/xview.h>
 #include <xview/canvas.h>
 #include <xview/panel.h>
+#include <xview/defaults.h>
 
 #ifdef SOUND
 #include <errno.h>
@@ -50,15 +51,15 @@
 #endif
 #endif
 
-#include "system.h"
 #endif
 
+#include "system.h"
 #include "signale.h"
 #include "ereignisse.h"
 #include "einaus.h"
 #include "xv_einaus.h"
 
-static char sccsid[] = "@(#)xv_ereignisse.c	1.31 7/16/94";
+static char sccsid[] = "@(#)xv_ereignisse.c	1.41 17 Dec 1994";
 
 
 #ifdef SOUND
@@ -173,7 +174,7 @@ struct
 } abspiel[MAX_MISCH];
 
 static int abspiel_anz;      /* Anzahl der zur Zeit abzuspielenden Sounds */
-static int audio_laeuft;     /* Audio abspielen? */
+static int audio_laeuft;     /* Audio aktiviert? */
 static int audio_deskriptor; /* Deskriptor fuer das Audio-Device */
 
 static Frame audio_frame; /* Fenster, zu dem audio_handler gehoert */
@@ -231,101 +232,6 @@ static char *fehler_text()
 #endif
 
 
-/*
-** ereignisse_darstellen
-**  fuehrt die Ausgabe der Signale aus
-**
-** Parameter:
-**  ereignisse: Feld aller Ereignisse, in dem die auszugebenen gesetzt sind
-**
-** Seiteneffekte:
-**  abspiel und abspiel_anz werden veraendert
-*/
-void ereignisse_darstellen(ereignisse)
-char ereignisse[ERG_ANZ];
-{
-#ifdef SOUND
-	/* Audio spielen? */
-	if (audio_laeuft)
-	{
-		char erg_nr;          /* Index fuer Ereignisse */
-		int neu;              /* neues Ereignis aufgetreten? */
-		void audio_handler(); /* Routine zum Abspielen */
-#ifdef AUDIO_CTL
-		int maske;            /* alte Signal-Maske */
-#endif
-
-#ifdef AUDIO_CTL
-		maske = sigblock(sigmask(SIGPOLL)); /* Signal blockieren */
-#endif
-
-		for (erg_nr = neu = 0; erg_nr < ERG_ANZ; erg_nr++)
-
-			/* Ereignis aufgetreten und passender Sound vorhanden? */
-			if (ereignisse[erg_nr] && sounds[erg_nr].laenge)
-			{
-#ifdef SOUND_NICHT_MISCHEN
-				/* erstes neu hinzugekommenes Ereignis? */
-				if (!neu)
-
-					/* vorige Sounds beenden */
-					abspiel_anz = 0;
-#endif
-
-				/* noch Platz fuer neue Sounds? */
-				if (abspiel_anz < MAX_MISCH)
-				{
-					/* Startposition des Sounds setzen */
-					abspiel[abspiel_anz].laenge = sounds[erg_nr].laenge;
-					abspiel[abspiel_anz].position = sounds[erg_nr].inhalt;
-
-					/* ein Sound mehr zum Abspielen */
-					abspiel_anz++;
-
-					/* neue Ereignisse aufgetreten, neue Sounds */
-					neu = 1;
-				}
-			}
-
-#ifdef AUDIO_CTL
-		sigsetmask(maske); /* Signal-Maske zuruecksetzen */
-#endif
-
-		audio_handler(neu); /* Sound starten */
-
-		/* nicht Blinken/Piepen ausloesen */
-		return;
-	}
-#endif
-
-	/* Falls Spieler selbst abgeschossen wurde, wird das Fenster kurz
-	   invertiert und ein Piep ausgegeben*/
-	if (ereignisse[ABGESCHOSSEN_ERG])
-		xv_set(flash_window,
-			WIN_ALARM,
-			0);
-
-	/* Falls Spieler selbst getroffen wurde, wird ein Piep ausgegeben */
-	else if (ereignisse[GETROFFEN_ERG])
-		xv_set(beep_window,
-			WIN_ALARM,
-			0);
-
-	/* Falls eine Aktion nicht moeglich war, wird ein Piep ausgegeben */
-	else if (ereignisse[AKTION_FEHLER_ERG])
-		xv_set(beep_window,
-			WIN_ALARM,
-			0);
-
-	/* Falls Spieler vom Server aus dem Spiel genommen wurde, wird
-	   ein Piep ausgegeben */
-	else if (ereignisse[SUSPENDIERT_ERG])
-		xv_set(beep_window,
-			WIN_ALARM,
-			0);
-}
-
-
 #ifdef SOUND
 #ifdef ULAW
 /*
@@ -335,7 +241,7 @@ char ereignisse[ERG_ANZ];
 ** Seiteneffekte:
 **  ulaw wird initialisiert
 */
-void init_ulaw()
+static void init_ulaw()
 {
 	int i, j, k, offset, von, bis;
 
@@ -386,7 +292,7 @@ Notify_client client;
 int signal;
 Notify_signal_mode mode;
 {
-	void audio_handler();
+	static void audio_handler();
 
 	audio_handler(0);
 
@@ -404,16 +310,16 @@ Notify_signal_mode mode;
 ** Seiteneffekte:
 **  abspiel und abspiel_anz werden veraendert
 */
-void audio_handler(neu)
+static void audio_handler(neu)
 int neu;
 {
 	static int audio_puffer_laenge = 0; /* Laenge der noch
 	                                       abzuspielenden Audio-Daten */
-
 #ifdef AUDIO_CTL
-	int maske; /* alte Signal-Maske */
+	int freigeben; /* muss SIGPOLL wieder freigegeben werden? */
 
-	maske = sigblock(sigmask(SIGPOLL)); /* Signal blockieren */
+	if (freigeben = !signal_blockiert(SIGPOLL))
+		signal_blockieren(SIGPOLL); /* Signal blockieren */
 #endif
 
 	for (;;)
@@ -519,7 +425,9 @@ int neu;
 			if (eof_zaehler != info.play.eof)
 			{
 				/* dann noch keine neuen Daten spielen */
-				sigsetmask(maske);
+				if (freigeben)
+					signal_freigeben(SIGPOLL); /* Signal wieder freigeben */
+
 				return;
 			}
 		}
@@ -555,7 +463,8 @@ int neu;
 			{
 #ifdef AUDIO_CTL
 				/* Signal freigeben */
-				sigsetmask(maske);
+				if (freigeben)
+					signal_freigeben(SIGPOLL);
 #else
 				/* xv_audio_handler aufrufen, wenn weitere Daten
 				   geschrieben werden koennen */
@@ -579,7 +488,8 @@ int neu;
 
 #ifdef AUDIO_CTL
 				/* Signal freigeben */
-				sigsetmask(maske);
+				if (freigeben)
+					signal_freigeben(SIGPOLL);
 #endif
 
 				/* fertig */
@@ -592,7 +502,8 @@ int neu;
 
 #ifdef AUDIO_CTL
 	/* Signal freigeben */
-	sigsetmask(maske);
+	if (freigeben)
+		signal_freigeben(SIGPOLL);
 #else
 	/* xv_audio_handler nicht mehr aufrufen */
 	notify_set_output_func(audio_frame, NULL, audio_deskriptor);
@@ -601,9 +512,109 @@ int neu;
 #endif
 
 
+/* bis hier lokaler Teil                       */
+/***********************************************/
+/* ab hier globaler Teil                       */
+
+
+/*
+** ereignisse_darstellen
+**  fuehrt die Ausgabe der Signale aus
+**
+** Parameter:
+**  ereignisse: Feld aller Ereignisse, in dem die auszugebenen gesetzt sind
+**
+** Seiteneffekte:
+**  abspiel und abspiel_anz werden veraendert
+*/
+void ereignisse_darstellen(ereignisse)
+char ereignisse[ERG_ANZ];
+{
+#ifdef SOUND
+	/* Audio spielen? */
+	if (audio_laeuft)
+	{
+		char erg_nr;          /* Index fuer Ereignisse */
+		int neu;              /* neues Ereignis aufgetreten? */
+		void audio_handler(); /* Routine zum Abspielen */
+#ifdef AUDIO_CTL
+		int freigeben;        /* muss SIGPOLL wieder freigegeben werden? */
+
+		if (freigeben = !signal_blockiert(SIGPOLL))
+			signal_blockieren(SIGPOLL); /* Signal blockieren */
+#endif
+
+		for (erg_nr = neu = 0; erg_nr < ERG_ANZ; erg_nr++)
+
+			/* Ereignis aufgetreten und passender Sound vorhanden? */
+			if (ereignisse[erg_nr] && sounds[erg_nr].laenge)
+			{
+#ifdef SOUND_NICHT_MISCHEN
+				/* erstes neu hinzugekommenes Ereignis? */
+				if (!neu)
+
+					/* vorige Sounds beenden */
+					abspiel_anz = 0;
+#endif
+
+				/* noch Platz fuer neue Sounds? */
+				if (abspiel_anz < MAX_MISCH)
+				{
+					/* Startposition des Sounds setzen */
+					abspiel[abspiel_anz].laenge = sounds[erg_nr].laenge;
+					abspiel[abspiel_anz].position = sounds[erg_nr].inhalt;
+
+					/* ein Sound mehr zum Abspielen */
+					abspiel_anz++;
+
+					/* neue Ereignisse aufgetreten, neue Sounds */
+					neu = 1;
+				}
+			}
+
+#ifdef AUDIO_CTL
+		if (freigeben)
+			signal_freigeben(SIGPOLL); /* Signal wieder freigeben */
+#endif
+
+		audio_handler(neu); /* Sound starten */
+
+		/* nicht Blinken/Piepen ausloesen */
+		return;
+	}
+#endif
+
+	/* Falls Spieler selbst abgeschossen wurde, wird das Fenster kurz
+	   invertiert und ein Piep ausgegeben*/
+	if (ereignisse[ABGESCHOSSEN_ERG])
+		xv_set(flash_window,
+			WIN_ALARM,
+			0);
+
+	/* Falls Spieler selbst getroffen wurde, wird ein Piep ausgegeben */
+	else if (ereignisse[GETROFFEN_ERG])
+		xv_set(beep_window,
+			WIN_ALARM,
+			0);
+
+	/* Falls eine Aktion nicht moeglich war, wird ein Piep ausgegeben */
+	else if (ereignisse[AKTION_FEHLER_ERG])
+		xv_set(beep_window,
+			WIN_ALARM,
+			0);
+
+	/* Falls Spieler vom Server aus dem Spiel genommen wurde, wird
+	   ein Piep ausgegeben */
+	else if (ereignisse[SUSPENDIERT_ERG])
+		xv_set(beep_window,
+			WIN_ALARM,
+			0);
+}
+
+
 /*
 ** xv_ereignisse_init
-**  Such nach xv-Objekten, in denen die Signale ausgegeben werden
+**  sucht nach XView-Objekten, in denen die Signale ausgegeben werden
 **
 ** Parameter:
 **  hauptframe: Fenster, von dem alle anderen Fenster abstammen
@@ -624,8 +635,8 @@ char **argv;
 	int i;            /* Index fuer Unterfenster */
 	Window subwindow; /* Unterfenster */
 #ifdef SOUND
-    static char *audio_verzeichnis; /* Verzeichnis, in dem die
-	                                   Sound-Dateien liegen */
+    static char audio_verzeichnis[64]; /* Verzeichnis, in dem die
+	                                      Sound-Dateien liegen */
 #endif
 
 	/* Pieps werden im Hauptfenster ausgeloest */
@@ -647,8 +658,8 @@ char **argv;
 
 gefunden:
 	;
-#ifdef SOUND
 
+#ifdef SOUND
 	/* Fenster fuer Audio-Signal-Handler merken */
 	audio_frame = hauptframe;
 
@@ -658,7 +669,13 @@ gefunden:
 
 		/* Parameter initialisieren */
 		audio_laeuft = 0;
-		audio_verzeichnis = DEFAULT_SOUND_DIR;
+
+		/* Name des Audio-Verzeichnisses abfragen und kopieren */
+		strncpy(audio_verzeichnis,
+			defaults_get_string("iMaze.sound.directory",
+			"IMaze.Sound.Directory", DEFAULT_SOUND_DIR),
+			sizeof audio_verzeichnis);
+		audio_verzeichnis[sizeof audio_verzeichnis - 1] = 0;
 
 		/* Kommandozeilenparameter auswerten */
 		for (ueberspringen = 1; ueberspringen < *argc &&
@@ -685,7 +702,9 @@ gefunden:
 					             suchen und spielen */
 						if (argv[ueberspringen][i] == 0)
 							ueberspringen++, i = 0;
-						audio_verzeichnis = argv[ueberspringen] + i;
+						strncpy(audio_verzeichnis, argv[ueberspringen] + i,
+							sizeof audio_verzeichnis);
+						audio_verzeichnis[sizeof audio_verzeichnis - 1] = 0;
 						i = 0;
 
 					case 's': /* Sounds spielen */
@@ -765,17 +784,6 @@ gefunden:
 			uebler_fehler(meldung, NULL);
 		}
 
-		/* Signale durch das Audio-Kontroll-Device zulassen */
-		if (ioctl(audio_ctl_deskriptor, I_SETSIG, S_MSG))
-		{
-			static char *meldung[] = { "iMaze - Sound Error", "",
-				"Can't enable signals on audio control device:",
-				NULL, NULL };
-
-			meldung[3] = fehler_text();
-			uebler_fehler(meldung, NULL);
-		}
-
 		/* Audio-Status abfragen */
 		if (ioctl(audio_ctl_deskriptor, AUDIO_GETINFO, &info))
 		{
@@ -853,15 +861,8 @@ gefunden:
 	{
 		struct audio_info info; /* Audio-Status */
 
-		/* Status abfragen */
-		if (ioctl(audio_deskriptor, AUDIO_GETINFO, &info))
-		{
-			static char *meldung[] = { "iMaze - Sound Error", "",
-				"Can't get audio info:", NULL, NULL };
-
-			meldung[3] = fehler_text();
-			uebler_fehler(meldung, NULL);
-		}
+		/* Status initialisieren */
+		AUDIO_INITINFO(&info);
 
 		/* Abtastrate setzen */
 		info.play.sample_rate = SAMPLING_RATE;
@@ -939,7 +940,7 @@ gefunden:
 					read(deskriptor, puffer, laenge);
 
 					/* au-Header vorhanden? */
-					if (!strncmp(puffer, ".snd", 4))
+					if (!strncmp((char *)puffer, ".snd", 4))
 					{
 						offset = (int)puffer[4] << 24 |
 							(int)puffer[5] << 16 |
@@ -1021,6 +1022,17 @@ gefunden:
 		/* Handler fuer Audio-Signal installieren */
 		notify_set_signal_func(audio_frame, xv_audio_handler,
 			SIGPOLL, NOTIFY_ASYNC);
+
+		/* Signale durch das Audio-Kontroll-Device zulassen */
+		if (ioctl(audio_ctl_deskriptor, I_SETSIG, S_MSG))
+		{
+			static char *meldung[] = { "iMaze - Sound Error", "",
+				"Can't enable signals on audio control device:",
+				NULL, NULL };
+
+			meldung[3] = fehler_text();
+			uebler_fehler(meldung, NULL);
+		}
 #endif
 	}
 #endif
@@ -1031,7 +1043,7 @@ gefunden:
 
 /*
 ** xv_ereignisse_ende
-**  Dummy-Routine fuer das Schliessen von Ausgabekanaelen etc.
+**  Routine fuer das Schliessen von Ausgabekanaelen etc.
 */
 void xv_ereignisse_ende()
 {
